@@ -158,6 +158,7 @@ static void
 ccsSetActivePluginList (CCSContext * context, CCSStringList list)
 {
     CCSPluginList l;
+    CCSPlugin     *plugin;
 
     for (l = context->plugins; l; l = l->next)
     {
@@ -167,7 +168,6 @@ ccsSetActivePluginList (CCSContext * context, CCSStringList list)
 
     for (; list; list = list->next)
     {
-	CCSPlugin *plugin;
 	plugin = ccsFindPlugin (context, list->data);
 
 	if (plugin)
@@ -176,6 +176,14 @@ ccsSetActivePluginList (CCSContext * context, CCSStringList list)
 	    pPrivate->active = TRUE;
 	}
     }
+
+    /* core plugin is always active */
+    plugin = ccsFindPlugin (context, "core");
+    if (plugin)
+    {
+	PLUGIN_PRIV (plugin);
+	pPrivate->active = TRUE;
+    }
 }
 
 CCSContext *
@@ -213,7 +221,7 @@ ccsContextNew (unsigned int *screens, unsigned int numScreens)
 }
 
 CCSPlugin *
-ccsFindPlugin (CCSContext * context, char *name)
+ccsFindPlugin (CCSContext * context, const char *name)
 {
     if (!name)
 	name = "";
@@ -231,7 +239,7 @@ ccsFindPlugin (CCSContext * context, char *name)
 }
 
 CCSSetting *
-ccsFindSetting (CCSPlugin * plugin, char *name,
+ccsFindSetting (CCSPlugin * plugin, const char *name,
 		Bool isScreen, unsigned int screenNum)
 {
     PLUGIN_PRIV (plugin);
@@ -324,7 +332,6 @@ groupAdd (CCSSetting * setting, CCSPluginPrivate * p)
 }
 
 void
-
 collateGroups (CCSPluginPrivate * p)
 {
     CCSSettingList l = p->settings;
@@ -388,6 +395,7 @@ ccsFreePlugin (CCSPlugin * p)
 
     ccsSettingListFree (pPrivate->settings, TRUE);
     ccsGroupListFree (pPrivate->groups, TRUE);
+    ccsStrExtensionListFree (pPrivate->stringExtensions, TRUE);
 
     if (pPrivate->xmlFile)
 	free (pPrivate->xmlFile);
@@ -417,6 +425,9 @@ ccsFreeSetting (CCSSetting * s)
     case TypeInt:
 	ccsIntDescListFree (s->info.forInt.desc, TRUE);
 	break;
+    case TypeString:
+	ccsStrRestrictionListFree (s->info.forString.restriction, TRUE);
+	break;
     case TypeList:
 	if (s->info.forList.listType == TypeInt)
 	    ccsIntDescListFree (s->info.forList.listInfo->
@@ -498,18 +509,9 @@ ccsFreePluginConflict (CCSPluginConflict * c)
 
     free (c->value);
 
-    if (c->plugins)
-	c->plugins = ccsPluginListFree (c->plugins, FALSE);
-}
+    ccsPluginListFree (c->plugins, FALSE);
 
-void
-ccsFreeActionConflict (CCSActionConflict * c)
-{
-    if (!c)
-	return;
-
-    if (c->settings)
-	c->settings = ccsSettingListFree (c->settings, FALSE);
+    free (c);
 }
 
 void
@@ -526,6 +528,8 @@ ccsFreeBackendInfo (CCSBackendInfo * b)
 
     if (b->longDesc)
 	free (b->longDesc);
+
+    free (b);
 }
 
 void
@@ -536,6 +540,38 @@ ccsFreeIntDesc (CCSIntDesc * i)
 
     if (i->name)
 	free (i->name);
+
+    free (i);
+}
+
+void
+ccsFreeStrRestriction (CCSStrRestriction * r)
+{
+    if (!r)
+	return;
+
+    if (r->name)
+	free (r->name);
+
+    if (r->value)
+	free (r->value);
+
+    free (r);
+}
+
+void
+ccsFreeStrExtension (CCSStrExtension *e)
+{
+    if (!e)
+	return;
+
+    if (e->basePlugin)
+	free (e->basePlugin);
+
+    ccsStringListFree (e->baseSettings, TRUE);
+    ccsStrRestrictionListFree (e->restriction, TRUE);
+
+    free (e);
 }
 
 static void *
@@ -551,25 +587,26 @@ openBackend (char *backend)
 	asprintf (&dlname, "%s/.compizconfig/backends/lib%s.so", 
 		  home, backend);
 	dlerror ();
-	dlhand = dlopen (dlname, RTLD_NOW | RTLD_NODELETE | RTLD_GLOBAL);
+	dlhand = dlopen (dlname, RTLD_NOW | RTLD_NODELETE | RTLD_LOCAL);
 	err = dlerror ();
     }
 
-    if (err || !dlhand)
+    if (!dlhand)
     {
-	free (dlname);
+        if (dlname) {
+	        free (dlname);
+        }
 	asprintf (&dlname, "%s/compizconfig/backends/lib%s.so", 
 		  LIBDIR, backend);
-	dlhand = dlopen (dlname, RTLD_NOW | RTLD_NODELETE | RTLD_GLOBAL);
+	dlhand = dlopen (dlname, RTLD_NOW | RTLD_NODELETE | RTLD_LOCAL);
 	err = dlerror ();
     }
 
     free (dlname);
 
-    if (err || !dlhand)
+    if (err)
     {
 	fprintf (stderr, "libccs: dlopen: %s\n", err);
-	return NULL;
     }
 
     return dlhand;
@@ -926,14 +963,14 @@ ccsSetMatch (CCSSetting * setting, const char *data)
 }
 
 Bool
-ccsSetAction (CCSSetting * setting, CCSSettingActionValue data)
+ccsSetKey (CCSSetting * setting, CCSSettingKeyValue data)
 {
-    if (setting->type != TypeAction)
+    if (setting->type != TypeKey)
 	return FALSE;
 
-    CCSSettingActionValue defValue = setting->defaultValue.value.asAction;
+    CCSSettingKeyValue defValue = setting->defaultValue.value.asKey;
 
-    Bool isDefault = ccsIsEqualAction (data, defValue);
+    Bool isDefault = ccsIsEqualKey (data, defValue);
 
     if (setting->isDefault && isDefault)
 	return TRUE;
@@ -944,32 +981,114 @@ ccsSetAction (CCSSetting * setting, CCSSettingActionValue data)
 	return TRUE;
     }
 
-    if (ccsIsEqualAction (setting->value->value.asAction, data))
+    if (ccsIsEqualKey (setting->value->value.asKey, data))
 	return TRUE;
 
     if (setting->isDefault)
 	copyFromDefault (setting);
 
-    if (setting->info.forAction.key)
+    setting->value->value.asKey.keysym = data.keysym;
+    setting->value->value.asKey.keyModMask = data.keyModMask;
+
+    setting->parent->context->changedSettings =
+	ccsSettingListAppend (setting->parent->context->changedSettings,
+			      setting);
+
+    return TRUE;
+}
+
+Bool
+ccsSetButton (CCSSetting * setting, CCSSettingButtonValue data)
+{
+    if (setting->type != TypeButton)
+	return FALSE;
+
+    CCSSettingButtonValue defValue = setting->defaultValue.value.asButton;
+
+    Bool isDefault = ccsIsEqualButton (data, defValue);
+
+    if (setting->isDefault && isDefault)
+	return TRUE;
+
+    if (!setting->isDefault && isDefault)
     {
-	setting->value->value.asAction.keysym = data.keysym;
-	setting->value->value.asAction.keyModMask = data.keyModMask;
+	ccsResetToDefault (setting);
+	return TRUE;
     }
 
-    if (setting->info.forAction.button)
+    if (ccsIsEqualButton (setting->value->value.asButton, data))
+	return TRUE;
+
+    if (setting->isDefault)
+	copyFromDefault (setting);
+
+    setting->value->value.asButton.button = data.button;
+    setting->value->value.asButton.buttonModMask = data.buttonModMask;
+    setting->value->value.asButton.edgeMask = data.edgeMask;
+
+    setting->parent->context->changedSettings =
+	ccsSettingListAppend (setting->parent->context->changedSettings,
+			      setting);
+
+    return TRUE;
+}
+
+Bool
+ccsSetEdge (CCSSetting * setting, unsigned int data)
+{
+    if (setting->type != TypeEdge)
+	return FALSE;
+
+    Bool isDefault = (data == setting->defaultValue.value.asEdge);
+
+    if (setting->isDefault && isDefault)
+	return TRUE;
+
+    if (!setting->isDefault && isDefault)
     {
-	setting->value->value.asAction.button = data.button;
-	setting->value->value.asAction.buttonModMask = data.buttonModMask;
+	ccsResetToDefault (setting);
+	return TRUE;
     }
 
-    if (setting->info.forAction.edge)
+    if (setting->value->value.asEdge == data)
+	return TRUE;
+
+    if (setting->isDefault)
+	copyFromDefault (setting);
+
+    setting->value->value.asEdge = data;
+
+    setting->parent->context->changedSettings =
+	ccsSettingListAppend (setting->parent->context->changedSettings,
+			      setting);
+
+    return TRUE;
+}
+
+Bool
+ccsSetBell (CCSSetting * setting, Bool data)
+{
+    if (setting->type != TypeBell)
+	return FALSE;
+
+    Bool isDefault = (data == setting->defaultValue.value.asBool);
+
+    if (setting->isDefault && isDefault)
+	return TRUE;
+
+    if (!setting->isDefault && isDefault)
     {
-	setting->value->value.asAction.edgeButton = data.edgeButton;
-	setting->value->value.asAction.edgeMask = data.edgeMask;
+	ccsResetToDefault (setting);
+	return TRUE;
     }
 
-    if (setting->info.forAction.bell)
-	setting->value->value.asAction.onBell = data.onBell;
+    if (setting->value->value.asBell == data)
+	return TRUE;
+
+    if (setting->isDefault)
+	copyFromDefault (setting);
+
+    setting->value->value.asBell = data;
 
     setting->parent->context->changedSettings =
 	ccsSettingListAppend (setting->parent->context->changedSettings,
@@ -1006,9 +1125,22 @@ ccsCompareLists (CCSSettingValueList l1, CCSSettingValueList l2,
 	    if (strcmp (l1->data->value.asMatch, l2->data->value.asMatch))
 		return FALSE;
 	    break;
-	case TypeAction:
-	    if (!ccsIsEqualAction
-		(l1->data->value.asAction, l2->data->value.asAction))
+	case TypeKey:
+	    if (!ccsIsEqualKey
+		(l1->data->value.asKey, l2->data->value.asKey))
+		return FALSE;
+	    break;
+	case TypeButton:
+	    if (!ccsIsEqualButton
+		(l1->data->value.asButton, l2->data->value.asButton))
+		return FALSE;
+	    break;
+	case TypeEdge:
+	    if (l1->data->value.asEdge != l2->data->value.asEdge)
+		return FALSE;
+	    break;
+	case TypeBell:
+	    if (l1->data->value.asBell != l2->data->value.asBell)
 		return FALSE;
 	    break;
 	case TypeColor:
@@ -1062,15 +1194,26 @@ ccsCopyList (CCSSettingValueList l1, CCSSetting * setting)
 	case TypeMatch:
 	    value->value.asMatch = strdup (l1->data->value.asMatch);
 	    break;
-	case TypeAction:
-	    memcpy (&value->value.asAction, &l1->data->value.asAction,
-		    sizeof (CCSSettingActionValue));
+	case TypeKey:
+	    memcpy (&value->value.asKey, &l1->data->value.asKey,
+		    sizeof (CCSSettingKeyValue));
+	    break;
+	case TypeButton:
+	    memcpy (&value->value.asButton, &l1->data->value.asButton,
+		    sizeof (CCSSettingButtonValue));
+	    break;
+	case TypeEdge:
+	    value->value.asEdge = l1->data->value.asEdge;
+	    break;
+	case TypeBell:
+	    value->value.asBell = l1->data->value.asBell;
 	    break;
 	case TypeColor:
 	    memcpy (&value->value.asColor, &l1->data->value.asColor,
 		    sizeof (CCSSettingColorValue));
 	    break;
 	default:
+	  /* FIXME If l2 != NULL, we leak l2 */
 	    free (value);
 	    return FALSE;
 	    break;
@@ -1152,8 +1295,17 @@ ccsSetValue (CCSSetting * setting, CCSSettingValue * data)
     case TypeMatch:
 	return ccsSetMatch (setting, data->value.asMatch);
 	break;
-    case TypeAction:
-	return ccsSetAction (setting, data->value.asAction);
+    case TypeKey:
+	return ccsSetKey (setting, data->value.asKey);
+	break;
+    case TypeButton:
+	return ccsSetButton (setting, data->value.asButton);
+	break;
+    case TypeEdge:
+	return ccsSetEdge (setting, data->value.asEdge);
+	break;
+    case TypeBell:
+	return ccsSetBell (setting, data->value.asBell);
 	break;
     case TypeList:
 	return ccsSetList (setting, data->value.asList);
@@ -1225,12 +1377,42 @@ ccsGetMatch (CCSSetting * setting, char **data)
 }
 
 Bool
-ccsGetAction (CCSSetting * setting, CCSSettingActionValue * data)
+ccsGetKey (CCSSetting * setting, CCSSettingKeyValue * data)
 {
-    if (setting->type != TypeAction)
+    if (setting->type != TypeKey)
 	return FALSE;
 
-    *data = setting->value->value.asAction;
+    *data = setting->value->value.asKey;
+    return TRUE;
+}
+
+Bool
+ccsGetButton (CCSSetting * setting, CCSSettingButtonValue * data)
+{
+    if (setting->type != TypeButton)
+	return FALSE;
+
+    *data = setting->value->value.asButton;
+    return TRUE;
+}
+
+Bool
+ccsGetEdge (CCSSetting * setting, unsigned int * data)
+{
+    if (setting->type != TypeEdge)
+	return FALSE;
+
+    *data = setting->value->value.asEdge;
+    return TRUE;
+}
+
+Bool
+ccsGetBell (CCSSetting * setting, Bool * data)
+{
+    if (setting->type != TypeBell)
+	return FALSE;
+
+    *data = setting->value->value.asBell;
     return TRUE;
 }
 
@@ -1275,7 +1457,9 @@ ccsGetActivePluginList (CCSContext * context)
     {
 	PLUGIN_PRIV (l->data);
 	if (pPrivate->active && strcmp (l->data->name, "ccp"))
+	{
 	    rv = ccsPluginListAppend (rv, l->data);
+	}
 
 	l = l->next;
     }
@@ -1310,12 +1494,17 @@ CCSStringList
 ccsGetSortedPluginStringList (CCSContext * context)
 {
     CCSPluginList ap = ccsGetActivePluginList (context);
-    CCSPluginList list = ap;
+    CCSPluginList list;
     CCSPlugin *p = NULL;
-    CCSStringList rv = NULL;
+    CCSStringList rv = ccsStringListAppend (NULL, strdup ("core"));
     PluginSortHelper *ph = NULL;
 
+    p = findPluginInList (ap, "core");
+    if (p)
+	ap = ccsPluginListRemove (ap, p, FALSE);
+
     int len = ccsPluginListLength (ap);
+    
     int i, j;
     /* TODO: conflict handling */
 
@@ -1323,7 +1512,7 @@ ccsGetSortedPluginStringList (CCSContext * context)
     if (!plugins)
 	return NULL;
 
-    for (i = 0; i < len; i++, list = list->next)
+    for (i = 0, list = ap; i < len; i++, list = list->next)
     {
 	plugins[i].plugin = list->data;
 	plugins[i].after = NULL;
@@ -1401,6 +1590,11 @@ ccsGetSortedPluginStringList (CCSContext * context)
 	    if (plugins[i].after)
 		continue;
 
+	    /* This is a special case to ensure that bench is the last plugin */
+	    if (len - removed > 1 &&
+		strcmp (plugins[i].plugin->name, "bench") == 0)
+		continue;
+
 	    found = TRUE;
 	    removed++;
 	    p = plugins[i].plugin;
@@ -1742,18 +1936,21 @@ ccsIsEqualColor (CCSSettingColorValue c1, CCSSettingColorValue c2)
 }
 
 Bool
-ccsIsEqualAction (CCSSettingActionValue c1, CCSSettingActionValue c2)
+ccsIsEqualKey (CCSSettingKeyValue c1, CCSSettingKeyValue c2)
+{
+    if (c1.keysym == c2.keysym && c1.keyModMask == c2.keyModMask)
+	return TRUE;
+
+    return FALSE;
+}
+
+Bool
+ccsIsEqualButton (CCSSettingButtonValue c1, CCSSettingButtonValue c2)
 {
     if (c1.button == c2.button               &&
 	c1.buttonModMask == c2.buttonModMask &&
-	c1.keysym == c2.keysym               &&
-	c1.keyModMask == c2.keyModMask       &&
-	c1.edgeMask == c2.edgeMask           &&
-	c1.edgeButton == c2.edgeButton       &&
-	((c1.onBell && c2.onBell) || (!c1.onBell && !c2.onBell)))
-    {
+	c1.edgeMask == c2.edgeMask)
 	return TRUE;
-    }
 
     return FALSE;
 }
@@ -2052,7 +2249,7 @@ ccsCanDisablePlugin (CCSContext * context, CCSPlugin * plugin)
 	    if (!ccsPluginIsActive (context, pl->data->name))
 		continue;
 
-	    pluginList = pl->data->requiresPlugin;
+	    pluginList = pl->data->requiresFeature;
 
 	    while (pluginList)
 	    {
@@ -2065,7 +2262,7 @@ ccsCanDisablePlugin (CCSContext * context, CCSPlugin * plugin)
 			if (conflict)
 			{
 			    conflict->value = strdup (sl->data);
-			    conflict->type = ConflictPluginNeeded;
+			    conflict->type = ConflictFeatureNeeded;
 			}
 		    }
 		    if (conflict)
@@ -2074,147 +2271,17 @@ ccsCanDisablePlugin (CCSContext * context, CCSPlugin * plugin)
 		}
 		pluginList = pluginList->next;
 	    }
-	    if (conflict)
-		list = ccsPluginConflictListAppend (list, conflict);
+	    
 	}
+	if (conflict)
+	    list = ccsPluginConflictListAppend (list, conflict);
+	conflict = NULL;
 	sl = sl->next;
     }
 
     return list;
 }
 
-static Bool
-conflictKey (CCSSettingActionValue a1, CCSSettingActionValue a2)
-{
-    if (a1.keysym == a2.keysym && a1.keyModMask == a2.keyModMask)
-	return TRUE;
-
-    return FALSE;
-}
-
-static Bool
-conflictButton (CCSSettingActionValue a1, CCSSettingActionValue a2)
-{
-    if (a1.button == a2.button && a1.buttonModMask == a2.buttonModMask)
-	return TRUE;
-
-    return FALSE;
-}
-
-static Bool
-conflictEdge (CCSSettingActionValue a1, CCSSettingActionValue a2)
-{
-    if (a1.edgeMask & a2.edgeMask)
-	return TRUE;
-
-    return FALSE;
-}
-
-
-CCSActionConflictList
-ccsCanSetAction (CCSContext * context, CCSSettingActionValue action)
-{
-    CCSActionConflictList rv = NULL;
-    CCSActionConflict *keyC, *buttonC, *edgeC;
-
-    keyC = calloc (1, sizeof (CCSActionConflict));
-    if (!keyC)
-	return NULL;
-
-    buttonC = calloc (1, sizeof (CCSActionConflict));
-    if (!buttonC)
-    {
-	free (keyC);
-	return NULL;
-    }
-
-    edgeC = calloc (1, sizeof (CCSActionConflict));
-    if (!edgeC)
-    {
-	free (keyC);
-	free (buttonC);
-	return NULL;
-    }
-
-    keyC->type = ConflictKey;
-    buttonC->type = ConflictButton;
-    edgeC->type = ConflictEdge;
-
-    CCSPluginList pl = context->plugins;
-    CCSSettingList sl = NULL;
-    CCSSetting *s;
-
-    while (pl)
-    {
-	PLUGIN_PRIV (pl->data);
-
-	if (!pPrivate->loaded)
-	    ccsLoadPluginSettings (pl->data);
-
-	sl = pPrivate->settings;
-
-	while (sl)
-	{
-	    s = sl->data;
-
-	    if (s->type == TypeAction)
-	    {
-		if (conflictKey (action, s->value->value.asAction))
-		    keyC->settings = ccsSettingListAppend (keyC->settings, s);
-
-		if (conflictButton (action, s->value->value.asAction))
-		    buttonC->settings =
-			ccsSettingListAppend (buttonC->settings, s);
-
-		if (conflictEdge (action, s->value->value.asAction))
-		    edgeC->settings =
-			ccsSettingListAppend (edgeC->settings, s);
-	    }
-	    else if (s->type == TypeList
-		     && s->info.forList.listType == TypeAction)
-	    {
-		CCSSettingValueList vl = s->value->value.asList;
-
-		while (vl)
-		{
-		    if (conflictKey (action, vl->data->value.asAction))
-			keyC->settings =
-			    ccsSettingListAppend (keyC->settings, s);
-
-		    if (conflictButton (action, vl->data->value.asAction))
-			buttonC->settings =
-			    ccsSettingListAppend (buttonC->settings, s);
-
-		    if (conflictEdge (action, vl->data->value.asAction))
-			edgeC->settings =
-			    ccsSettingListAppend (edgeC->settings, s);
-
-		    vl = vl->next;
-		}
-	    }
-	    sl = sl->next;
-	}
-	pl = pl->next;
-    }
-
-    if (keyC->settings)
-	rv = ccsActionConflictListAppend (rv, keyC);
-    else
-	free (keyC);
-
-    if (buttonC->settings)
-	rv = ccsActionConflictListAppend (rv, buttonC);
-    else
-	free (buttonC);
-
-    if (edgeC->settings)
-	rv = ccsActionConflictListAppend (rv, edgeC);
-    else
-	free (edgeC);
-
-    return rv;
-}
-
 CCSStringList
 ccsGetExistingProfiles (CCSContext * context)
 {
@@ -2266,7 +2333,7 @@ addBackendInfo (CCSBackendInfoList * bl, char *file)
 
     dlerror ();
 
-    dlhand = dlopen (file, RTLD_LAZY);
+    dlhand = dlopen (file, RTLD_LAZY | RTLD_LOCAL);
     err = dlerror ();
     if (err || !dlhand)
 	return;
@@ -2384,7 +2451,9 @@ ccsGetExistingBackends ()
 }
 
 Bool
-ccsExportToFile (CCSContext * context, const char * fileName)
+ccsExportToFile (CCSContext *context,
+		 const char *fileName,
+		 Bool       skipDefaults)
 {
     IniDictionary *exportFile;
     CCSPluginList p;
@@ -2409,6 +2478,9 @@ ccsExportToFile (CCSContext * context, const char * fileName)
 	{
 	    setting = s->data;
 
+	    if (skipDefaults && setting->isDefault)
+		continue;
+
 	    if (setting->isScreen)
 		asprintf (&keyName, "s%d_%s", 
 			  setting->screenNum, setting->name);
@@ -2425,13 +2497,29 @@ ccsExportToFile (CCSContext * context, const char * fileName)
 		ccsIniSetInt (exportFile, plugin->name, keyName,
 			      setting->value->value.asInt);
 		break;
+	    case TypeFloat:
+		ccsIniSetFloat (exportFile, plugin->name, keyName,
+				setting->value->value.asFloat);
+		break;
 	    case TypeString:
 		ccsIniSetString (exportFile, plugin->name, keyName,
 				 setting->value->value.asString);
 		break;
-	    case TypeAction:
-		ccsIniSetAction (exportFile, plugin->name, keyName,
-				 setting->value->value.asAction);
+	    case TypeKey:
+		ccsIniSetKey (exportFile, plugin->name, keyName,
+			      setting->value->value.asKey);
+		break;
+	    case TypeButton:
+		ccsIniSetButton (exportFile, plugin->name, keyName,
+				 setting->value->value.asButton);
+		break;
+	    case TypeEdge:
+		ccsIniSetEdge (exportFile, plugin->name, keyName,
+			       setting->value->value.asEdge);
+		break;
+	    case TypeBell:
+		ccsIniSetBell (exportFile, plugin->name, keyName,
+			       setting->value->value.asBell);
 		break;
 	    case TypeColor:
 		ccsIniSetColor (exportFile, plugin->name, keyName,
@@ -2460,7 +2548,9 @@ ccsExportToFile (CCSContext * context, const char * fileName)
 }
 
 Bool
-ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
+ccsImportFromFile (CCSContext *context,
+		   const char *fileName,
+		   Bool       overwriteNonDefault)
 {
     IniDictionary *importFile;
     CCSPluginList p;
@@ -2468,6 +2558,13 @@ ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
     CCSPlugin *plugin;
     CCSSetting *setting;
     char *keyName;
+    FILE *fp;
+
+    /* check if the file exists first */
+    fp = fopen (fileName, "r");
+    if (!fp)
+	return FALSE;
+    fclose (fp);
 
     importFile = iniparser_new ((char *) fileName);
     if (!importFile)
@@ -2484,7 +2581,7 @@ ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
 	for (s = pPrivate->settings; s; s = s->next)
 	{
 	    setting = s->data;
-	    if (!setting->isDefault && !overwrite)
+	    if (!setting->isDefault && !overwriteNonDefault)
 		continue;
 
 	    if (setting->isScreen)
@@ -2513,6 +2610,15 @@ ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
 			ccsSetInt (setting, value);
 		}
 		break;
+	    case TypeFloat:
+		{
+		    float value;
+
+		    if (ccsIniGetFloat (importFile, plugin->name,
+					keyName, &value))
+			ccsSetFloat (setting, value);
+		}
+		break;
 	    case TypeString:
 		{
 		    char *value;
@@ -2522,13 +2628,40 @@ ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
 			ccsSetString (setting, value);
 		}
 		break;
-	    case TypeAction:
+	    case TypeKey:
 		{
-		    CCSSettingActionValue value;
+		    CCSSettingKeyValue value;
 
-		    if (ccsIniGetAction (importFile, plugin->name,
+		    if (ccsIniGetKey (importFile, plugin->name,
+				      keyName, &value))
+			ccsSetKey (setting, value);
+		}
+		break;
+	    case TypeButton:
+		{
+		    CCSSettingButtonValue value;
+
+		    if (ccsIniGetButton (importFile, plugin->name,
 					 keyName, &value))
-			ccsSetAction (setting, value);
+			ccsSetButton (setting, value);
+		}
+		break;
+	    case TypeEdge:
+		{
+		    unsigned int value;
+
+		    if (ccsIniGetEdge (importFile, plugin->name,
+				       keyName, &value))
+			ccsSetEdge (setting, value);
+		}
+		break;
+	    case TypeBell:
+		{
+		    Bool value;
+
+		    if (ccsIniGetBell (importFile, plugin->name,
+				       keyName, &value))
+			ccsSetBell (setting, value);
 		}
 		break;
 	    case TypeColor:
@@ -2567,8 +2700,6 @@ ccsImportFromFile (CCSContext * context, const char * fileName, Bool overwrite)
 	}
     }
 
-    ccsWriteSettings (context);
-
     ccsIniClose (importFile);
 
     return TRUE;
@@ -2627,3 +2758,13 @@ Bool ccsSettingIsReadOnly (CCSSetting *setting)
     return FALSE;
 }
 
+CCSStrExtensionList ccsGetPluginStrExtensions (CCSPlugin *plugin)
+{
+    PLUGIN_PRIV (plugin);
+
+    if (!pPrivate->loaded)
+	ccsLoadPluginSettings (plugin);
+
+    return pPrivate->stringExtensions;
+}
+
